-- BuffCheck_Data.lua
-- by Mithrandir1210

--[[ TO-DO LIST:

	- Change strings in BuffMeta to SpellIDs
	 
]]

local BuffCheck_Name, bc = ...
	-- BuffCheck_Name = "BuffCheck"
	-- BuffCheck = {}
	
-- SAVED VARIABLE TABLE
BuffCheck_ActiveSettings = {} -- Used for settings/options

-- COLOR PREFIXES --
bc.Colors = {
	Red = "|cFFFF0000",
	Blue = "|cFF0000FF",
	Green = "|cFF008000",
	Yellow = "|cFFFFFF00",
	Brown = "|cFF663300",
	Gray = "|cFF808080",
	White = "|cFFFFFFFF",
	Black = "|cFF000000",
	Aquablue = "|cFF00B8E6"
}

-- DEFAULT SETTINGS --
bc.Defaults = {
	FontColor = bc.Colors.Aquablue,
	DisableOutput = false,
	AnnounceMissing = false,
	CheckAfterReadycheck = false,
	DisplaySettingsOnLoad = true
}

bc.NumPlayers = nil

-- BUFF TABLES --=============================================================

--[[ **NOTE TO DEVS:
 
	- To add buffs to check for, entries must be made in the buff tables
		below (1), the 'MissingMeta' table (2), and the 'bc.Buffs' table (3).
			
	- Do not forget to add an entry in 'bc.Buffs.List' as well.
]]

-- Buffs to check for

local Food = {"Well Fed"} -- Modify to differentiate between +125 and +100
local GreaterFlasks = {
    "Greater Draenic Agility Flask",
    "Greater Draenic Intellect Flask",
    "Greater Draenic Strength Flask",
    "Greater Draenic Stamina Flask"
}
local Stats = {
    "Mark of the Wild",
	"Legacy of the Emperor",
	"Legacy of the White Tiger",
	"Blessing of Kings",
	"Lone Wolf: Power of the Primates",
	"Bark of the Wild",
	"Blessing of Kongs",
	"Embrace of the Shale Spider",
	"Strength of the Earth"
}
local Stam = {
    "Power Word: Fortitude",
    "Blood Pact",
    "Commanding Shout",
    "Lone Wolf: Fortitude of the Bear",
    "Invigorating Roar",
    "Sturdiness",
    "Savage Vigor",
    "Qiraji Fortitude"
}
local Ap = {
    "Trueshot Aura",
	 "Battle Shout",
	 "Horn of Winter"
}
local Sp = {
    "Arcane Brilliance",
	 "Dalaran Brilliance",
	 "Dark Intent",
	 "Lone Wolf: Wisdom of the Serpent",
	 "Serpent's Cunning",
	 "Qiraji Fortitude",
	 "Still Water"
}
local Mastery = {
    "Grace of Air",
    "Moonkin Aura",
    "Blessing of Might",
    "Power of the Grave",
    "Lone Wolf: Grace of the Cat",
    "Roar of Courage",
    "Keen Senses",
    "Spirit Beast Blessing",
    "Plainswalking"
}
local Haste = {
    "Unholy Aura",
    "Mind Quickening",
    "Swiftblade's Cunning",
    "Grace of Air",
    "Lone Wolf: Haste of the Hyena",
    "Cackling Howl",
    "Savage Vigor",
    "Energizing Spores",
    "Speed of the Swarm"
}
local Crit = {
    "Leader of the Pack",
    "Arcane Brilliance",
    "Dalaran Brilliance",
    "Legacy of the White Tiger",
    "Lone Wolf: Ferocity of the Raptor",
    "Terrifying Roar",
    "Fearless Roar",
    "Strength of the Pack",
    "Embrace of the Shale Spider",
    "Still Water",
    "Furious Howl"
}
local Multi = {
    "Dark Intent",
    "Windflurry",
    "Mind Quickening",
    "Swiftblade's Cunning",
    "Lone Wolf: Quickness of the Dragonhawk",
    "Sonic Focus",
    "Wild Strength",
    "Double Bite",
    "Spry Attacks",
    "Breath of the Winds"
}
local Vers = {
    "Mark of the Wild",
    "Unholy Aura",
    "Sanctity Aura",
    "Inspiring Presence",
    "Lone Wolf: Versatility of the Ravager",
    "Tenacity",
    "Indomitable",
    "Wild Strength",
    "Defensive Quills",
    "Chitinous Armor",
    "Grace",
    "Strength of the Earth"
}
local AugmentRune = {
    "Stout Augmentation",
    "Focus Augmentation",
    "Hyper Augmentation"
				 
}
--[[ Other Buffs (not implemented yet)
local RegularFlasks = {
    "Draenic Agility Flask",
    "Draenic Intellect Flask",
    "Draenic Stamina Flask"
]]

-- Players Missing Buffs
local MissingMeta = {
	Food = {},
	GreaterFlasks = {},
	Stats = {},
	Stam = {},
	Ap = {},
	Sp = {},
	Mastery = {},
	Haste = {},
	Crit = {},
	Multi = {},
	Vers = {},
	AugmentRune = {}
	-- Other Missing Buffs (not implemented yet)
	--RegularFlask = {}
}

-------------------------------------------------------------------------------
-- Returns whether a buff was found on a unit (boolean)
-- Checked against a specific buff table (i.e. 'TableName').
-------------------------------------------------------------------------------
local IsBuffFound = function (Unit, TableName)
	
	local i=1
	local j=1
	local Buff = (select(1,UnitBuff(Unit,j)))
	
	while TableName[i] do
		j = 1
		Buff = (select(1,UnitBuff(Unit,j)))
		
		while Buff do
			if TableName[i] == Buff then
				return true
			end
			j = j + 1
			Buff = (select(1,UnitBuff(Unit,j)))
		end
		i = i + 1
	end
	
	return false
	
end

-------------------------------------------------------------------------------
-- Clears all missing buff tables by setting them to nil (void).
-------------------------------------------------------------------------------
local ClearTables = function ()
		    
    for k,v in pairs(MissingMeta) do
		for kk, vv in pairs(MissingMeta[k]) do
			MissingMeta[k][kk] = nil
		end
	end
end

-------------------------------------------------------------------------------
-- Returns whether all buffs are found on the player or group (boolean).
-------------------------------------------------------------------------------
local AllBuffsFound = function ()

	local Result = true

	for k,v in pairs(MissingMeta) do
		for kk, vv in pairs(MissingMeta[k]) do
			if MissingMeta[k][kk] then
				Result = false
			end
		end
	end
	
	return Result

end

-------------------------------------------------------------------------------
-- Holds all settings and functions relating to each buff type/category (TABLE).
-------------------------------------------------------------------------------
bc.Buffs = {
	
	-- MUST MATCH Buff Table Names
	List = {
		"Food",
		"GreaterFlasks",
		"Stats",
		"Stam",
		"Ap",
		"Sp",
		"Mastery",
		"Haste",
		"Crit",
		"Multi",
		"Vers",
		"AugmentRune"
	},
	
	Clear = function ()
		ClearTables()
	end,
	
	AllBuffsFound = function ()
		return AllBuffsFound()
	end,
	
	Food = {
		Enable = true,
		Name = "<Food>",
		BuffTbl = Food,
		MissingTbl = MissingMeta.Food,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Food.MissingTbl[1] then
				Msg = bc.Buffs.Food.Name..": "..table.concat(bc.Buffs.Food.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Food.BuffTbl) then
				table.insert(bc.Buffs.Food.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},
	
	GreaterFlasks = {
		Enable = true,
		Name = "<Greater Flask>",
		BuffTbl = GreaterFlasks,
		MissingTbl = MissingMeta.GreaterFlasks,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.GreaterFlasks.MissingTbl[1] then
				Msg = bc.Buffs.GreaterFlasks.Name..": "..table.concat(bc.Buffs.GreaterFlasks.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.GreaterFlasks.BuffTbl) then
				table.insert(bc.Buffs.GreaterFlasks.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},
	
	Stats = {
		Enable = true,
		Name = "<Stats>",
		BuffTbl = Stats,
		MissingTbl = MissingMeta.Stats,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Stats.MissingTbl[1] then
				Msg = bc.Buffs.Stats.Name..": "..table.concat(bc.Buffs.Stats.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Stats.BuffTbl) then
				table.insert(bc.Buffs.Stats.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},

	Stam = {
		Enable = true,
		Name = "<Stamina>",
		BuffTbl = Stam,
		MissingTbl = MissingMeta.Stam,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Stam.MissingTbl[1] then
				Msg = bc.Buffs.Stam.Name..": "..table.concat(bc.Buffs.Stam.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Stam.BuffTbl) then
				table.insert(bc.Buffs.Stam.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},

	Ap = {
		Enable = true,
		Name = "<Attack Power>",
		BuffTbl = Ap,
		MissingTbl = MissingMeta.Ap,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Ap.MissingTbl[1] then
				Msg = bc.Buffs.Ap.Name..": "..table.concat(bc.Buffs.Ap.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Ap.BuffTbl) then
				table.insert(bc.Buffs.Ap.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},

	Sp = {
		Enable = true,
		Name = "<Spell Power>",
		BuffTbl = Sp,
		MissingTbl = MissingMeta.Sp,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Sp.MissingTbl[1] then
				Msg = bc.Buffs.Sp.Name..": "..table.concat(bc.Buffs.Sp.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Sp.BuffTbl) then
				table.insert(bc.Buffs.Sp.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},

	Mastery = {
		Enable = true,
		Name = "<Mastery>",
		BuffTbl = Mastery,
		MissingTbl = MissingMeta.Mastery,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Mastery.MissingTbl[1] then
				Msg = bc.Buffs.Mastery.Name..": "..table.concat(bc.Buffs.Mastery.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Mastery.BuffTbl) then
				table.insert(bc.Buffs.Mastery.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},

	Haste = {
		Enable = true,
		Name = "<Haste>",
		BuffTbl = Haste,
		MissingTbl = MissingMeta.Haste,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Haste.MissingTbl[1] then
				Msg = bc.Buffs.Haste.Name..": "..table.concat(bc.Buffs.Haste.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Haste.BuffTbl) then
				table.insert(bc.Buffs.Haste.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},

	Crit = {
		Enable = true,
		Name = "<Critical Strike>",
		BuffTbl = Crit,
		MissingTbl = MissingMeta.Crit,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Crit.MissingTbl[1] then
				Msg = bc.Buffs.Crit.Name..": "..table.concat(bc.Buffs.Crit.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Crit.BuffTbl) then
				table.insert(bc.Buffs.Crit.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},

	Multi = {
		Enable = true,
		Name = "<Multistrike>",
		BuffTbl = Multi,
		MissingTbl = MissingMeta.Multi,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Multi.MissingTbl[1] then
				Msg = bc.Buffs.Multi.Name..": "..table.concat(bc.Buffs.Multi.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Multi.BuffTbl) then
				table.insert(bc.Buffs.Multi.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},
	
	Vers = {
		Enable = true,
		Name = "<Versatility>",
		BuffTbl = Vers,
		MissingTbl = MissingMeta.Vers,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.Vers.MissingTbl[1] then
				Msg = bc.Buffs.Vers.Name..": "..table.concat(bc.Buffs.Vers.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.Vers.BuffTbl) then
				table.insert(bc.Buffs.Vers.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	},

	AugmentRune = {
		Enable = true,
		Name = "<Augment Rune>",
		BuffTbl = AugmentRune,
		MissingTbl = MissingMeta.AugmentRune,
		
		DisplayMissing = function (Channel)
			
			local Msg = nil
			if bc.Buffs.AugmentRune.MissingTbl[1] then
				Msg = bc.Buffs.AugmentRune.Name..": "..table.concat(bc.Buffs.AugmentRune.MissingTbl, ", ")
			end
			if Msg then
				bc.Display(Msg,Channel)
			end
		end,
		
		Check = function(Unit)
			if not IsBuffFound(Unit, bc.Buffs.AugmentRune.BuffTbl) then
				table.insert(bc.Buffs.AugmentRune.MissingTbl, (select(1,UnitName(Unit))) )
			end
		end
	}
	
}
